@@ -468,58 +468,6 @@ out:
         return (gchar **) g_ptr_array_free (opt_array, FALSE);
 }
 
-static const gchar *
-engine_from_locale (void)
-{
-        const gchar *locale;
-        const gchar *locale_engine[][2] = {
-                { "as_IN", "m17n:as:phonetic" },
-                { "bn_IN", "m17n:bn:inscript" },
-                { "gu_IN", "m17n:gu:inscript" },
-                { "hi_IN", "m17n:hi:inscript" },
-                { "ja_JP", "anthy" },
-                { "kn_IN", "m17n:kn:kgp" },
-                { "ko_KR", "hangul" },
-                { "mai_IN", "m17n:mai:inscript" },
-                { "ml_IN", "m17n:ml:inscript" },
-                { "mr_IN", "m17n:mr:inscript" },
-                { "or_IN", "m17n:or:inscript" },
-                { "pa_IN", "m17n:pa:inscript" },
-                { "sd_IN", "m17n:sd:inscript" },
-                { "ta_IN", "m17n:ta:tamil99" },
-                { "te_IN", "m17n:te:inscript" },
-                { "zh_CN", "pinyin" },
-                { "zh_HK", "cangjie3" },
-                { "zh_TW", "chewing" },
-        };
-        gint i;
-
-        locale = setlocale (LC_CTYPE, NULL);
-        if (!locale)
-                return NULL;
-
-        for (i = 0; i < G_N_ELEMENTS (locale_engine); ++i)
-                if (g_str_has_prefix (locale, locale_engine[i][0]))
-                        return locale_engine[i][1];
-
-        return NULL;
-}
-
-static void
-add_ibus_sources_from_locale (GSettings *settings)
-{
-        const gchar *locale_engine;
-        GVariantBuilder builder;
-
-        locale_engine = engine_from_locale ();
-        if (!locale_engine)
-                return;
-
-        init_builder_with_sources (&builder, settings);
-        g_variant_builder_add (&builder, "(ss)", INPUT_SOURCE_TYPE_IBUS, locale_engine);
-        g_settings_set_value (settings, KEY_INPUT_SOURCES, g_variant_builder_end (&builder));
-}
-
 static void
 convert_ibus (GSettings *settings)
 {
@@ -913,64 +861,29 @@ append_options (gchar **a,
 }
 
 static void
-strip_xkb_option (gchar       **options,
-                  const gchar  *prefix)
-{
-        guint last;
-        gchar **p = options;
-
-        if (!p)
-                return;
-
-        while (*p) {
-                if (g_str_has_prefix (*p, prefix)) {
-                        last = g_strv_length (options) - 1;
-                        g_free (*p);
-                        *p = options[last];
-                        options[last] = NULL;
-                }
-                p += 1;
-        }
-}
-
-static gchar *
-prepare_xkb_options (GsdKeyboardManager *manager,
-                     guint               n_sources,
-                     gchar             **extra_options)
+add_xkb_options (GsdKeyboardManager *manager,
+                 XkbRF_VarDefsRec   *xkb_var_defs,
+                 gchar             **extra_options)
 {
         gchar **options;
         gchar **settings_options;
-        gchar  *options_str;
 
         settings_options = g_settings_get_strv (manager->priv->input_sources_settings,
                                                 KEY_KEYBOARD_OPTIONS);
         options = append_options (settings_options, extra_options);
         g_strfreev (settings_options);
 
-        /* We might set up different layouts in different groups - see
-         * replace_layout_and_variant(). But we don't want the X
-         * server group switching feature to actually switch
-         * them. Regularly, if we have at least two input sources,
-         * gnome-shell will tell us to switch input source at that
-         * point so we fix that automatically. But when there's only
-         * one source, gnome-shell short circuits as an optimization
-         * and doesn't call us so we can't set the group switching XKB
-         * option in the first place otherwise the X server's switch
-         * will take effect and we get a broken configuration. */
-        if (n_sources < 2)
-                strip_xkb_option (options, "grp:");
+        free (xkb_var_defs->options);
+        xkb_var_defs->options = build_xkb_options_string (options);
 
-        options_str = build_xkb_options_string (options);
         g_strfreev (options);
-
-        return options_str;
 }
 
 static void
 apply_xkb_settings (GsdKeyboardManager *manager,
                     const gchar        *layout,
                     const gchar        *variant,
-                    gchar              *options)
+                    gchar             **options)
 {
         XkbRF_RulesRec *xkb_rules;
         XkbRF_VarDefsRec *xkb_var_defs;
@@ -978,9 +891,7 @@ apply_xkb_settings (GsdKeyboardManager *manager,
 
         gnome_xkb_info_get_var_defs (&rules_file_path, &xkb_var_defs);
 
-        free (xkb_var_defs->options);
-        xkb_var_defs->options = options;
-
+        add_xkb_options (manager, xkb_var_defs, options);
         replace_layout_and_variant (manager, xkb_var_defs, layout, variant);
 
         gdk_error_trap_push ();
@@ -1092,8 +1003,7 @@ apply_input_sources_settings (GSettings          *settings,
         }
 
  exit:
-        apply_xkb_settings (manager, layout, variant,
-                            prepare_xkb_options (manager, n_sources, options));
+        apply_xkb_settings (manager, layout, variant, options);
         maybe_return_from_set_input_source (manager);
         g_variant_unref (sources);
         g_free (layout);
@@ -1503,12 +1413,8 @@ maybe_create_initial_settings (GsdKeyboardManager *manager)
 
         /* if we still don't have anything do some educated guesses */
         sources = g_settings_get_value (settings, KEY_INPUT_SOURCES);
-        if (g_variant_n_children (sources) < 1) {
+        if (g_variant_n_children (sources) < 1)
                 get_sources_from_xkb_config (manager);
-#ifdef HAVE_IBUS
-                add_ibus_sources_from_locale (settings);
-#endif
-        }
         g_variant_unref (sources);
 
         options = g_settings_get_strv (settings, KEY_KEYBOARD_OPTIONS);
@@ -1517,6 +1423,32 @@ maybe_create_initial_settings (GsdKeyboardManager *manager)
         g_strfreev (options);
 }
 
+static void
+localed_proxy_ready (GObject      *source,
+                     GAsyncResult *res,
+                     gpointer      data)
+{
+        GsdKeyboardManager *manager = data;
+        GDBusProxy *proxy;
+        GError *error = NULL;
+
+        proxy = g_dbus_proxy_new_finish (res, &error);
+        if (!proxy) {
+                if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
+                        g_error_free (error);
+                        return;
+                }
+                g_warning ("Failed to contact localed: %s", error->message);
+                g_error_free (error);
+                goto out;
+        }
+
+        manager->priv->localed = proxy;
+        maybe_create_initial_settings (manager);
+out:
+        apply_input_sources_settings (manager->priv->input_sources_settings, NULL, 0, manager);
+}
+
 static void
 set_input_source_return (GDBusMethodInvocation *invocation)
 {
@@ -1580,12 +1512,10 @@ handle_dbus_method_call (GDBusConnection       *connection,
 
         if (g_str_equal (method_name, "SetInputSource")) {
                 if (priv->invocation) {
-#ifdef HAVE_IBUS
                         /* This can only happen if there's an
                          * ibus_bus_set_global_engine_async() call
                          * going on. */
                         g_cancellable_cancel (priv->ibus_cancellable);
-#endif
                         g_clear_pointer (&priv->invocation, set_input_source_return);
                         priv->pending_ops = 0;
                 }
@@ -1667,33 +1597,6 @@ register_manager_dbus (GsdKeyboardManager *manager)
                    manager);
 }
 
-static void
-localed_proxy_ready (GObject      *source,
-                     GAsyncResult *res,
-                     gpointer      data)
-{
-        GsdKeyboardManager *manager = data;
-        GDBusProxy *proxy;
-        GError *error = NULL;
-
-        proxy = g_dbus_proxy_new_finish (res, &error);
-        if (!proxy) {
-                if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
-                        g_error_free (error);
-                        return;
-                }
-                g_warning ("Failed to contact localed: %s", error->message);
-                g_error_free (error);
-                goto out;
-        }
-
-        manager->priv->localed = proxy;
-        maybe_create_initial_settings (manager);
-out:
-        apply_input_sources_settings (manager->priv->input_sources_settings, NULL, 0, manager);
-        register_manager_dbus (manager);
-}
-
 static gboolean
 start_keyboard_idle_cb (GsdKeyboardManager *manager)
 {
@@ -1733,6 +1636,7 @@ start_keyboard_idle_cb (GsdKeyboardManager *manager)
                           G_CALLBACK (apply_input_sources_settings), manager);
 
 	install_xkb_filter (manager);
+        register_manager_dbus (manager);
 
         gnome_settings_profile_end (NULL);
 
