@@ -29,7 +29,6 @@
 #include <glib/gi18n.h>
 #include <gdk/gdkx.h>
 #include <X11/extensions/XTest.h>
-#include <X11/extensions/dpms.h>
 #include <canberra-gtk.h>
 
 #define GNOME_DESKTOP_USE_UNSTABLE_API
@@ -1151,15 +1150,6 @@ disable_builtin_screensaver (gpointer unused)
 guint
 gsd_power_enable_screensaver_watchdog (void)
 {
-        int dummy;
-
-        /* Make sure that Xorg's DPMS extension never gets in our
-         * way. The defaults are now applied in Fedora 20 from
-         * being "0" by default to being "600" by default */
-        gdk_error_trap_push ();
-        if (DPMSQueryExtension(GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), &dummy, &dummy))
-                DPMSSetTimeouts (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), 0, 0, 0);
-        gdk_error_trap_pop_ignored ();
         return g_timeout_add_seconds (XSCREENSAVER_WATCHDOG_TIMEOUT,
                                       disable_builtin_screensaver,
                                       NULL);
@@ -1179,7 +1169,7 @@ get_primary_output (GnomeRRScreen *rr_screen)
 
         for (i = 0; outputs[i] != NULL; i++) {
                 if (gnome_rr_output_is_connected (outputs[i]) &&
-                    gnome_rr_output_is_laptop (outputs[i]) &&
+                    gnome_rr_output_is_builtin_display (outputs[i]) &&
                     gnome_rr_output_get_backlight_min (outputs[i]) >= 0 &&
                     gnome_rr_output_get_backlight_max (outputs[i]) > 0) {
                         output = outputs[i];
@@ -1196,19 +1186,21 @@ backlight_set_mock_value (gint value)
 {
 	const char *filename;
 	char *contents;
+	GError *error = NULL;
 
-	g_debug ("Settings mock brightness: %d", value);
+	g_debug ("Setting mock brightness: %d", value);
 
 	filename = "GSD_MOCK_brightness";
 	contents = g_strdup_printf ("%d", value);
-	g_file_set_contents (filename, contents, -1, NULL);
+	if (!g_file_set_contents (filename, contents, -1, NULL))
+		g_warning ("Setting mock brightness failed: %s", error->message);
+	g_clear_error (&error);
 	g_free (contents);
 }
 
 static gint64
 backlight_get_mock_value (const char *argument)
 {
-	const char *filename;
 	char *contents;
 	gint64 ret;
 
@@ -1217,14 +1209,9 @@ backlight_get_mock_value (const char *argument)
 		return GSD_MOCK_MAX_BRIGHTNESS;
 	}
 
-	if (g_str_equal (argument, "get-brightness")) {
-		filename = "GSD_MOCK_brightness";
-		ret = GSD_MOCK_DEFAULT_BRIGHTNESS;
-	} else {
-		g_assert_not_reached ();
-	}
+	g_assert (g_str_equal (argument, "get-brightness"));
 
-	if (g_file_get_contents (filename, &contents, NULL, NULL)) {
+	if (g_file_get_contents ("GSD_MOCK_brightness", &contents, NULL, NULL)) {
 		ret = g_ascii_strtoll (contents, NULL, 0);
 		g_free (contents);
 		g_debug ("Returning mock brightness: %"G_GINT64_FORMAT, ret);
@@ -1700,6 +1687,44 @@ randr_output_is_on (GnomeRROutput *output)
         return gnome_rr_crtc_get_current_mode (crtc) != NULL;
 }
 
+#ifdef GSD_MOCK
+static void
+mock_monitor_changed (GFileMonitor     *monitor,
+		      GFile            *file,
+		      GFile            *other_file,
+		      GFileMonitorEvent event_type,
+		      gpointer          user_data)
+{
+	GnomeRRScreen *screen = (GnomeRRScreen *) user_data;
+
+	g_debug ("Mock screen configuration changed");
+	g_signal_emit_by_name (G_OBJECT (screen), "changed");
+}
+
+static void
+screen_destroyed (gpointer  user_data,
+		  GObject  *where_the_object_was)
+{
+	g_object_unref (G_OBJECT (user_data));
+}
+#endif /* GSD_MOCK */
+
+void
+watch_external_monitor (GnomeRRScreen *screen)
+{
+#ifdef GSD_MOCK
+	GFile *file;
+	GFileMonitor *monitor;
+
+	file = g_file_new_for_commandline_arg ("GSD_MOCK_EXTERNAL_MONITOR");
+	monitor = g_file_monitor (file, G_FILE_MONITOR_NONE, NULL, NULL);
+	g_object_unref (file);
+	g_signal_connect (monitor, "changed",
+			  G_CALLBACK (mock_monitor_changed), screen);
+	g_object_weak_ref (G_OBJECT (screen), screen_destroyed, monitor);
+#endif /* GSD_MOCK */
+}
+
 gboolean
 external_monitor_is_connected (GnomeRRScreen *screen)
 {
@@ -1712,22 +1737,26 @@ external_monitor_is_connected (GnomeRRScreen *screen)
 	if (g_file_get_contents ("GSD_MOCK_EXTERNAL_MONITOR", &mock_external_monitor_contents, NULL, NULL)) {
 		if (mock_external_monitor_contents[0] == '1') {
 			g_free (mock_external_monitor_contents);
+			g_debug ("Mock external monitor is on");
 			return TRUE;
 		} else if (mock_external_monitor_contents[0] == '0') {
 			g_free (mock_external_monitor_contents);
+			g_debug ("Mock external monitor is off");
 			return FALSE;
 		}
 
-		g_warning ("Unhandled value for GSD_MOCK_EXTERNAL_MONITOR contents: %s", mock_external_monitor_contents);
+		g_error ("Unhandled value for GSD_MOCK_EXTERNAL_MONITOR contents: %s", mock_external_monitor_contents);
 		g_free (mock_external_monitor_contents);
 	}
 #endif /* GSD_MOCK */
 
+	g_assert (screen != NULL);
+
         /* see if we have more than one screen plugged in */
         outputs = gnome_rr_screen_list_outputs (screen);
         for (i = 0; outputs[i] != NULL; i++) {
                 if (randr_output_is_on (outputs[i]) &&
-                    !gnome_rr_output_is_laptop (outputs[i]))
+                    !gnome_rr_output_is_builtin_display (outputs[i]))
                         return TRUE;
         }
 
