@@ -25,7 +25,7 @@ import gsdpowerenums
 import dbus
 
 from gi.repository import Gio
-
+from gi.repository import GLib
 
 class PowerPluginTest(gsdtestcase.GSDTestCase):
     '''Test the power plugin'''
@@ -84,7 +84,11 @@ class PowerPluginTest(gsdtestcase.GSDTestCase):
         self.plugin_log_write = open(os.path.join(self.workdir, 'plugin_power.log'), 'wb')
         # avoid painfully long delays of actions for tests
         env = os.environ.copy()
+        # Disable the use of the PolicyKit helper for brightness
         env['GSD_DISABLE_BACKLIGHT_HELPER'] = '1'
+        # Disable PulseAudio output from libcanberra
+        env['CANBERRA_DRIVER'] = 'null'
+
         self.daemon = subprocess.Popen(
             [os.path.join(builddir, 'gsd-test-power')],
             # comment out this line if you want to see the logs in real time
@@ -165,25 +169,23 @@ class PowerPluginTest(gsdtestcase.GSDTestCase):
         return self.obj_session_presence_props.Get('org.gnome.SessionManager.Presence', 'status')
 
     def get_brightness(self):
-        f = open('GSD_MOCK_brightness', 'r')
-        ret = f.read()
-        f.close()
+        try:
+            (success, ret) = GLib.file_get_contents ('GSD_MOCK_brightness')
+        except:
+            return gsdpowerconstants.GSD_MOCK_DEFAULT_BRIGHTNESS
         return int(ret)
 
     def set_has_external_monitor(self, external):
-        f = open('GSD_MOCK_EXTERNAL_MONITOR', 'w')
         if external:
-            f.write('1')
+            val = '1'
         else:
-            f.write('0')
-        f.close ()
-
-        os.kill(self.daemon.pid, signal.SIGUSR2)
+            val = '0'
+        GLib.file_set_contents ('GSD_MOCK_EXTERNAL_MONITOR', val)
 
     def check_for_logout(self, timeout):
         '''Check that logout is requested.
 
-        Fail after the tiven timeout.
+        Fail after the given timeout.
         '''
         # check that it request suspend
         while timeout > 0:
@@ -193,7 +195,7 @@ class PowerPluginTest(gsdtestcase.GSDTestCase):
             try:
                 log = self.session_log.read()
             except IOError:
-                break
+                continue
 
             if log and (b'GsmManager: requesting logout' in log):
                 break
@@ -213,7 +215,7 @@ class PowerPluginTest(gsdtestcase.GSDTestCase):
     def check_for_suspend(self, timeout):
         '''Check that Suspend() or Hibernate() is requested.
 
-        Fail after the tiven timeout.
+        Fail after the given timeout.
         '''
         # check that it request suspend
         while timeout > 0:
@@ -223,13 +225,25 @@ class PowerPluginTest(gsdtestcase.GSDTestCase):
             try:
                 log = self.logind.stdout.read()
             except IOError:
-                break
+                continue
 
             if log and (b' Suspend ' in log or b' Hibernate ' in log):
                 break
         else:
             self.fail('timed out waiting for logind Suspend() call')
 
+    def check_for_uninhibited(self):
+        '''Check that the lid inhibitor has been dropped.
+
+        Fail after the given timeout.
+        '''
+        # check that it requested uninhibition
+        log = self.plugin_log.read()
+
+        if 'uninhibiting lid close' in log:
+            return
+        self.fail('timed out waiting for lid uninhibition')
+
     def check_no_suspend(self, seconds):
         '''Check that no Suspend or Hibernate is requested in the given time'''
 
@@ -342,7 +356,7 @@ class PowerPluginTest(gsdtestcase.GSDTestCase):
         # the locked screen saver
         self.reset_idle_timer()
         self.check_unblank(2)
-        self.assertTrue(self.get_brightness() == gsdpowerconstants.GSD_MOCK_DEFAULT_BRIGHTNESS , 'incorrect unblanked brightness')
+        self.assertTrue(self.get_brightness() == gsdpowerconstants.GSD_MOCK_DEFAULT_BRIGHTNESS , 'incorrect unblanked brightness (%d != %d)' % (self.get_brightness(), gsdpowerconstants.GSD_MOCK_DEFAULT_BRIGHTNESS))
 
         # Check for no blank before the normal blank timeout
         self.check_no_blank(gsdpowerconstants.SCREENSAVER_TIMEOUT_BLANK - 4)
@@ -368,7 +382,6 @@ class PowerPluginTest(gsdtestcase.GSDTestCase):
         self.settings_session['idle-delay'] = 10
         self.reset_idle_timer()
         time.sleep(5)
-        os.kill(self.session.pid, signal.SIGUSR2)
         self.assertEqual(self.get_status(), gsdpowerenums.GSM_PRESENCE_STATUS_AVAILABLE)
         time.sleep(10)
         self.assertEqual(self.get_status(), gsdpowerenums.GSM_PRESENCE_STATUS_IDLE)
@@ -377,7 +390,6 @@ class PowerPluginTest(gsdtestcase.GSDTestCase):
         self.settings_session['idle-delay'] = 5
         self.reset_idle_timer()
         time.sleep(2)
-        os.kill(self.session.pid, signal.SIGUSR2)
         self.assertEqual(self.get_status(), gsdpowerenums.GSM_PRESENCE_STATUS_AVAILABLE)
         time.sleep(5)
         self.assertEqual(self.get_status(), gsdpowerenums.GSM_PRESENCE_STATUS_IDLE)
@@ -385,6 +397,8 @@ class PowerPluginTest(gsdtestcase.GSDTestCase):
     def test_idle_time_reset_on_resume(self):
         '''Check that the IDLETIME is reset when resuming'''
 
+        self.settings_screensaver['lock-enabled'] = False
+
         # Go idle
         self.settings_session['idle-delay'] = 5
         self.assertEqual(self.get_status(), gsdpowerenums.GSM_PRESENCE_STATUS_AVAILABLE)
@@ -458,7 +472,8 @@ class PowerPluginTest(gsdtestcase.GSDTestCase):
 
         # Drop the inhibit and see whether we suspend
         self.obj_session_mgr.Uninhibit(dbus.UInt32(inhibit_id))
-        self.check_for_suspend(5)
+        # At this point logind should suspend for us
+        self.settings_screensaver['lock-enabled'] = False
 
     def test_blank_on_lid_close(self):
         '''Check that we do blank on lid closing, if the machine will not suspend'''
@@ -477,7 +492,7 @@ class PowerPluginTest(gsdtestcase.GSDTestCase):
 
         # Drop the inhibit and see whether we suspend
         self.obj_session_mgr.Uninhibit(dbus.UInt32(inhibit_id))
-        self.check_for_suspend(5)
+        # At this point logind should suspend for us
 
     def test_unblank_on_lid_open(self):
         '''Check that we do unblank on lid opening, if the machine will not suspend'''
@@ -520,7 +535,10 @@ class PowerPluginTest(gsdtestcase.GSDTestCase):
 
         # Wait and check we're not idle, but dimmed
         self.check_dim(gsdpowerconstants.MINIMUM_IDLE_DIM_DELAY)
-        self.assertTrue(self.get_brightness() == dim_level, 'incorrect dim brightness')
+        # Give time for the brightness to change
+        time.sleep(2)
+        level = self.get_brightness();
+        self.assertTrue(level == dim_level, 'incorrect dim brightness (%d != %d)' % (level, dim_level))
 
         self.assertEqual(self.get_status(), gsdpowerenums.GSM_PRESENCE_STATUS_AVAILABLE)
 
@@ -540,7 +558,7 @@ class PowerPluginTest(gsdtestcase.GSDTestCase):
         time.sleep(1)
 
         # And check that we have the pre-dim brightness
-        self.assertTrue(self.get_brightness() == gsdpowerconstants.GSD_MOCK_DEFAULT_BRIGHTNESS , 'incorrect unblanked brightness')
+        self.assertTrue(self.get_brightness() == gsdpowerconstants.GSD_MOCK_DEFAULT_BRIGHTNESS , 'incorrect unblanked brightness (%d != %d)' % (self.get_brightness(), gsdpowerconstants.GSD_MOCK_DEFAULT_BRIGHTNESS))
 
     def test_no_suspend_lid_close(self):
         '''Check that we don't suspend on lid close with an external monitor'''
@@ -559,7 +577,10 @@ class PowerPluginTest(gsdtestcase.GSDTestCase):
 
         # Unplug the external monitor
         self.set_has_external_monitor(False)
-        self.check_for_suspend (10)
+        # Wait for the safety timer + 3 seconds
+        time.sleep (gsdpowerconstants.LID_CLOSE_SAFETY_TIMEOUT + 3)
+        # Check that we're uninhibited
+        self.check_for_uninhibited()
 
     def test_action_critical_battery(self):
         '''action on critical battery'''
@@ -589,25 +610,6 @@ class PowerPluginTest(gsdtestcase.GSDTestCase):
         # verify notification
         self.assertRegex(notify_log, b'[0-9.]+ Notify "Power" 0 "battery-.*" ".*battery critical.*"')
 
-    def test_action_critical_battery_on_start(self):
-        '''action on critical battery on startup'''
-
-        # add a fake battery with 2%/1 minute charge to upower
-        bat_path = self.obj_upower.AddDischargingBattery('mock_BAT', 'Mock Bat', 2.0, 60)
-        obj_bat = self.system_bus_con.get_object('org.freedesktop.UPower', bat_path)
-        self.obj_upower.EmitSignal('', 'DeviceAdded', 's', [bat_path],
-                                   dbus_interface='org.freedesktop.DBus.Mock')
-
-        time.sleep(5)
-
-        # we should have gotten a notification now
-        notify_log = self.p_notify.stdout.read()
-
-        self.check_for_suspend(5)
-
-        # verify notification
-        self.assertRegex(notify_log, b'[0-9.]+ Notify "Power" 0 "battery-.*" ".*battery critical.*"')
-
     def test_action_multiple_batteries(self):
         '''critical actions for multiple batteries'''
 
