@@ -32,8 +32,10 @@
 #include <locale.h>
 
 #include <glib.h>
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
+#include <libnotify/notify.h>
 #include <X11/Xatom.h>
 #include <X11/extensions/XTest.h>
 #include <X11/keysym.h>
@@ -47,6 +49,7 @@
 #include "gnome-settings-profile.h"
 #include "gsd-wacom-manager.h"
 #include "gsd-wacom-device.h"
+#include "gsd-wacom-oled.h"
 #include "gsd-wacom-osd-window.h"
 
 #define GSD_WACOM_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GSD_TYPE_WACOM_MANAGER, GsdWacomManagerPrivate))
@@ -58,6 +61,7 @@
 #define KEY_AREA                "area"
 #define KEY_DISPLAY             "display"
 #define KEY_KEEP_ASPECT         "keep-aspect"
+#define KEY_LAST_CALIBRATED_RESOLUTION "last-calibrated-resolution"
 
 /* Stylus and Eraser settings */
 #define KEY_BUTTON_MAPPING      "buttonmapping"
@@ -65,13 +69,18 @@
 #define KEY_PRESSURECURVE       "pressurecurve"
 
 /* Button settings */
-#define KEY_ACTION_TYPE         "action-type"
-#define KEY_CUSTOM_ACTION       "custom-action"
+#define KEY_ACTION_TYPE            "action-type"
+#define KEY_CUSTOM_ACTION          "custom-action"
 #define KEY_CUSTOM_ELEVATOR_ACTION "custom-elevator-action"
+#define OLED_LABEL                 "oled-label"
 
 /* See "Wacom Pressure Threshold" */
 #define DEFAULT_PRESSURE_THRESHOLD 27
 
+#define CALIBRATION_NOTIFICATION_TIMEOUT 15000
+#define SHOW_CALIBRATION_TIMEOUT          2000
+#define UNKNOWN_DEVICE_NOTIFICATION_TIMEOUT 15000
+
 struct GsdWacomManagerPrivate
 {
         guint start_idle_id;
@@ -79,20 +88,28 @@ struct GsdWacomManagerPrivate
         guint device_added_id;
         guint device_removed_id;
         GHashTable *devices; /* key = GdkDevice, value = GsdWacomDevice */
-        GList *rr_screens;
+        GnomeRRScreen *rr_screen;
+        GHashTable *warned_devices;
 
         /* button capture */
-        GSList *screens;
+        GdkScreen *screen;
         int      opcode;
 
         /* Help OSD window */
         GtkWidget *osd_window;
+
+        guint notification_timeout_src_id;
+        NotifyNotification *calibration_notification;
+        GsdWacomDevice *calibration_device;
 };
 
 static void     gsd_wacom_manager_class_init  (GsdWacomManagerClass *klass);
 static void     gsd_wacom_manager_init        (GsdWacomManager      *wacom_manager);
 static void     gsd_wacom_manager_finalize    (GObject              *object);
 
+static void     on_notification_closed (NotifyNotification *notification,
+                                        GsdWacomManager    *manager);
+
 G_DEFINE_TYPE (GsdWacomManager, gsd_wacom_manager, G_TYPE_OBJECT)
 
 static gpointer manager_object = NULL;
@@ -691,7 +708,7 @@ reset_pad_buttons (GsdWacomDevice *device)
 
 	XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdev);
 
-	/* Reset all the LEDs */
+	/* Reset all the LEDs and OLEDs*/
 	buttons = gsd_wacom_device_get_buttons (device);
 	for (l = buttons; l != NULL; l = l->next) {
 		GsdWacomTabletButton *button = l->data;
@@ -699,16 +716,36 @@ reset_pad_buttons (GsdWacomDevice *device)
                     button->status_led != GSD_WACOM_NO_LED) {
                         set_led (device, button, 1);
                 }
-        }
+		if (button->has_oled) {
+			char *label;
+			label = g_settings_get_string (button->settings, OLED_LABEL);
+			set_oled (device, button->id, label);
+			g_free (label);
+		}
+	}
         g_list_free (buttons);
 }
 
+static void
+gsettings_oled_changed (GSettings *settings,
+			gchar *key,
+			GsdWacomTabletButton *button)
+{
+	GsdWacomDevice *device;
+	char *label;
+
+	label = g_settings_get_string (settings, OLED_LABEL);
+	device = g_object_get_data (G_OBJECT (button->settings), "parent-device");
+	set_oled (device, button->id, label);
+}
+
 static void
 set_wacom_settings (GsdWacomManager *manager,
 		    GsdWacomDevice  *device)
 {
 	GsdWacomDeviceType type;
 	GSettings *settings;
+	GList *buttons, *l;
 
 	g_debug ("Applying settings for device '%s' (type: %s)",
 		 gsd_wacom_device_get_tool_name (device),
@@ -745,7 +782,19 @@ set_wacom_settings (GsdWacomManager *manager,
 
 		id = get_device_id (device);
 		reset_pad_buttons (device);
-		grab_button (id, TRUE, manager->priv->screens);
+		grab_button (id, TRUE, manager->priv->screen);
+
+		buttons = gsd_wacom_device_get_buttons (device);
+		for (l = buttons; l != NULL; l = l->next) {
+			GsdWacomTabletButton *button = l->data;
+			if (button->has_oled) {
+				g_signal_connect (G_OBJECT (button->settings), "changed::oled-label",
+						  G_CALLBACK (gsettings_oled_changed), button);
+				g_object_set_data (G_OBJECT (button->settings), "parent-device", device);
+			}
+		}
+		g_list_free (buttons);
+
 		return;
 	}
 
@@ -790,6 +839,9 @@ wacom_settings_changed (GSettings      *settings,
 		    type != WACOM_TYPE_PAD &&
 		    type != WACOM_TYPE_TOUCH)
 			set_absolute (device, g_settings_get_boolean (settings, key));
+	} else if (g_str_equal (key, KEY_LAST_CALIBRATED_RESOLUTION)) {
+                /* We do nothing as this is used only for
+                   checking whether calibration is needed*/
 	} else if (g_str_equal (key, KEY_AREA)) {
 		if (type != WACOM_TYPE_CURSOR &&
 		    type != WACOM_TYPE_PAD &&
@@ -951,6 +1003,24 @@ osd_window_update_viewable (GsdWacomManager      *manager,
 	return TRUE;
 }
 
+static void
+notify_unknown_device (GsdWacomManager *manager, const gchar *device_name)
+{
+        gchar *msg_body;
+        NotifyNotification *notification;
+
+        msg_body = g_strdup_printf (_("The \"%s\" tablet may not work as expected."), device_name);
+        notification = notify_notification_new (_("Unknown Tablet Connected"), msg_body, NULL);
+        notify_notification_set_timeout (notification, UNKNOWN_DEVICE_NOTIFICATION_TIMEOUT);
+        notify_notification_set_urgency (notification, NOTIFY_URGENCY_NORMAL);
+        notify_notification_set_app_name (notification, _("Wacom Settings"));
+        notify_notification_show (notification, NULL);
+
+        g_signal_connect (notification, "closed", G_CALLBACK (on_notification_closed), manager);
+
+        g_free (msg_body);
+}
+
 static void
 device_added_cb (GdkDeviceManager *device_manager,
                  GdkDevice        *gdk_device,
@@ -958,8 +1028,26 @@ device_added_cb (GdkDeviceManager *device_manager,
 {
 	GsdWacomDevice *device;
 	GSettings *settings;
+	const gchar *device_name;
 
 	device = gsd_wacom_device_new (gdk_device);
+	device_name = gsd_wacom_device_get_name (device);
+
+        if (gsd_wacom_device_is_fallback (device) &&
+            gsd_wacom_device_get_device_type (device) != WACOM_TYPE_TOUCH &&
+            device_name != NULL) {
+                GHashTable *warned_devices;
+
+                warned_devices = manager->priv->warned_devices;
+
+                if (!g_hash_table_contains (warned_devices, device_name)) {
+                        g_warning ("No definition for  '%s' found in the tablet database. Using a fallback one.",
+                                   device_name);
+                        g_hash_table_insert (warned_devices, g_strdup (device_name), NULL);
+                        notify_unknown_device (manager, device_name);
+                }
+        }
+
 	if (gsd_wacom_device_get_device_type (device) == WACOM_TYPE_INVALID) {
 		g_object_unref (device);
 		return;
@@ -1339,10 +1427,11 @@ static gboolean
 gsd_wacom_manager_idle_cb (GsdWacomManager *manager)
 {
 	GList *devices, *l;
-	GSList *ls;
 
         gnome_settings_profile_start (NULL);
 
+        manager->priv->warned_devices = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+
         manager->priv->devices = g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL, g_object_unref);
 
         set_devicepresence_handler (manager);
@@ -1353,11 +1442,9 @@ gsd_wacom_manager_idle_cb (GsdWacomManager *manager)
         g_list_free (devices);
 
         /* Start filtering the button events */
-        for (ls = manager->priv->screens; ls != NULL; ls = ls->next) {
-                gdk_window_add_filter (gdk_screen_get_root_window (ls->data),
-                                       (GdkFilterFunc) filter_button_events,
-                                       manager);
-        }
+        gdk_window_add_filter (gdk_screen_get_root_window (manager->priv->screen),
+                               (GdkFilterFunc) filter_button_events,
+                               manager);
 
         gnome_settings_profile_end (NULL);
 
@@ -1366,6 +1453,167 @@ gsd_wacom_manager_idle_cb (GsdWacomManager *manager)
         return FALSE;
 }
 
+static gboolean
+check_need_for_calibration (GsdWacomManager *manager,
+                            GsdWacomDevice  *device)
+{
+        GSettings *settings;
+        GVariant *variant;
+        gint width, height;
+        GdkScreen *screen;
+        gint monitor;
+        GdkRectangle geometry;
+
+        width = -1;
+        height = -1;
+
+        screen = gdk_screen_get_default ();
+        monitor = gsd_wacom_device_get_display_monitor (device);
+
+	if (monitor < 0) {
+		geometry.width = gdk_screen_get_width (screen);
+		geometry.height = gdk_screen_get_height (screen);
+	} else {
+		gdk_screen_get_monitor_geometry (screen, monitor, &geometry);
+	}
+
+        settings = gsd_wacom_device_get_settings (device);
+
+        variant = g_settings_get_value (settings, KEY_LAST_CALIBRATED_RESOLUTION);
+
+        g_variant_get (variant, "(ii)", &width, &height);
+
+        g_debug ("Last calibrated resolution: %d, %d", width, height);
+
+        return width == -1 || width != geometry.width ||
+                height == -1 || height != geometry.height;
+}
+
+static void
+remove_notification (GsdWacomManager *manager)
+{
+        g_clear_object (&manager->priv->calibration_notification);
+        manager->priv->calibration_device = NULL;
+}
+
+static void
+on_notification_closed (NotifyNotification *notification,
+                        GsdWacomManager    *manager)
+{
+        if (notification == manager->priv->calibration_notification)
+                remove_notification (manager);
+        else
+                g_object_unref (notification);
+}
+
+static void
+on_notification_action (NotifyNotification *notification,
+                        gchar              *action,
+                        gpointer            data)
+{
+        gboolean success;
+        gchar *command;
+        const gchar *device_name;
+        GError *error = NULL;
+        GsdWacomManager *manager = GSD_WACOM_MANAGER (data);
+
+        device_name = gsd_wacom_device_get_name (manager->priv->calibration_device);
+
+        if (g_strcmp0 (action, "run-calibration") == 0) {
+                command = g_strdup_printf ("gnome-control-center wacom \"%s\" run-calibration",
+                                           device_name);
+                success = g_spawn_command_line_async (command, &error);
+                if (!success) {
+                        g_warning ("Failure launching gnome-control-center: %s",
+                                   error->message);
+                        g_clear_error (&error);
+                }
+                g_free (command);
+        }
+
+        notify_notification_close (manager->priv->calibration_notification, NULL);
+        remove_notification (manager);
+}
+
+static void
+remove_calibration_notification (GsdWacomManager *manager)
+{
+        GsdWacomManagerPrivate *priv;
+
+        priv = manager->priv;
+
+        if (priv->calibration_notification) {
+                notify_notification_close (priv->calibration_notification, NULL);
+                g_clear_object (&priv->calibration_notification);
+        }
+        if (manager->priv->notification_timeout_src_id > 0) {
+		g_source_remove (manager->priv->notification_timeout_src_id);
+		manager->priv->notification_timeout_src_id = 0;
+	}
+	manager->priv->calibration_device = NULL;
+}
+
+static gboolean
+notify_need_for_calibration_real (gpointer data)
+{
+        GsdWacomManager *manager = GSD_WACOM_MANAGER (data);
+        GsdWacomManagerPrivate *priv;
+        gchar *msg_body = NULL;
+
+        priv = manager->priv;
+
+        if (priv->calibration_notification) {
+                notify_notification_close (priv->calibration_notification, NULL);
+                g_clear_object (&priv->calibration_notification);
+        }
+        msg_body = g_strdup_printf (_("Tablet %s needs to be calibrated."),
+                                    gsd_wacom_device_get_name (priv->calibration_device));
+        priv->calibration_notification = notify_notification_new (_("Calibration needed"),
+                                                                  msg_body,
+                                                                  NULL);
+
+        notify_notification_set_app_name (priv->calibration_notification,
+                                          _("Wacom Settings"));
+        notify_notification_set_timeout (priv->calibration_notification,
+                                         CALIBRATION_NOTIFICATION_TIMEOUT);
+        notify_notification_set_urgency (priv->calibration_notification,
+                                         NOTIFY_URGENCY_NORMAL);
+        /* TRANSLATORS: launches the calibration screen for the
+           tablet in question */
+        notify_notification_add_action (priv->calibration_notification,
+                                        "run-calibration",
+                                        _("Calibrate"),
+                                        on_notification_action,
+                                        manager,
+                                        NULL);
+
+        g_signal_connect (priv->calibration_notification,
+                          "closed",
+                          G_CALLBACK (on_notification_closed),
+                          manager);
+
+        notify_notification_show (priv->calibration_notification, NULL);
+
+        priv->notification_timeout_src_id = 0;
+        g_free (msg_body);
+        return FALSE;
+}
+
+/* We need to have the notifications being shown from a
+ * timeout function since when the monitor's size changes
+ * we may get several calls to on_screen_changed_cb and
+ * in the first call, the monitor's size is still not updated.
+ */
+static void
+notify_need_for_calibration (GsdWacomManager *manager,
+                             GsdWacomDevice  *device)
+{
+        manager->priv->calibration_device = device;
+        manager->priv->notification_timeout_src_id = g_timeout_add (SHOW_CALIBRATION_TIMEOUT,
+                                                                    notify_need_for_calibration_real,
+                                                                    manager);
+}
+
 /*
  * The monitors-changed signal is emitted when the number, size or
  * position of the monitors attached to the screen change.
@@ -1400,6 +1648,14 @@ on_screen_changed_cb (GnomeRRScreen *rr_screen,
 		if (type != WACOM_TYPE_TOUCH) {
 			if (gsd_wacom_device_is_screen_tablet (device) == FALSE)
 				set_keep_aspect (device, g_settings_get_boolean (settings, KEY_KEEP_ASPECT));
+			else if (type == WACOM_TYPE_STYLUS) {
+				remove_calibration_notification (manager);
+				if (check_need_for_calibration (manager, device)) {
+					notify_need_for_calibration (manager,
+								     device);
+				}
+			}
+
 			set_area (device, g_settings_get_value (settings, KEY_AREA));
 		}
 		set_display (device, g_settings_get_value (settings, KEY_DISPLAY));
@@ -1408,39 +1664,33 @@ on_screen_changed_cb (GnomeRRScreen *rr_screen,
 }
 
 static void
-init_screens (GsdWacomManager *manager)
+init_screen (GsdWacomManager *manager)
 {
-        GdkDisplay *display;
-        int i;
+        GError *error = NULL;
+        GdkScreen *screen;
+        GnomeRRScreen *rr_screen;
 
-        display = gdk_display_get_default ();
-        for (i = 0; i < gdk_display_get_n_screens (display); i++) {
-                GError *error = NULL;
-                GdkScreen *screen;
-                GnomeRRScreen *rr_screen;
-
-                screen = gdk_display_get_screen (display, i);
-                if (screen == NULL) {
-                        continue;
-                }
-                manager->priv->screens = g_slist_append (manager->priv->screens, screen);
-
-		/*
-		 * We also keep a list of GnomeRRScreen to monitor changes such as rotation
-		 * which are not reported by Gdk's "monitors-changed" callback
-		 */
-		rr_screen = gnome_rr_screen_new (screen, &error);
-		if (rr_screen == NULL) {
-			g_warning ("Failed to create GnomeRRScreen: %s", error->message);
-			g_error_free (error);
-			continue;
-		}
-		manager->priv->rr_screens = g_list_prepend (manager->priv->rr_screens, rr_screen);
-		g_signal_connect (rr_screen,
-				  "changed",
-				  G_CALLBACK (on_screen_changed_cb),
-			          manager);
+        screen = gdk_screen_get_default ();
+        if (screen == NULL) {
+                return;
         }
+        manager->priv->screen = screen;
+
+        /*
+         * We keep GnomeRRScreen to monitor changes such as rotation
+         * which are not reported by Gdk's "monitors-changed" callback
+         */
+        rr_screen = gnome_rr_screen_new (screen, &error);
+        if (rr_screen == NULL) {
+                g_warning ("Failed to create GnomeRRScreen: %s", error->message);
+                g_error_free (error);
+                return;
+        }
+        manager->priv->rr_screen = rr_screen;
+        g_signal_connect (rr_screen,
+                          "changed",
+                          G_CALLBACK (on_screen_changed_cb),
+                          manager);
 }
 
 gboolean
@@ -1459,7 +1709,7 @@ gsd_wacom_manager_start (GsdWacomManager *manager,
                 return TRUE;
         }
 
-        init_screens (manager);
+        init_screen (manager);
 
         manager->priv->start_idle_id = g_idle_add ((GSourceFunc) gsd_wacom_manager_idle_cb, manager);
 
@@ -1472,7 +1722,6 @@ void
 gsd_wacom_manager_stop (GsdWacomManager *manager)
 {
         GsdWacomManagerPrivate *p = manager->priv;
-        GSList *ls;
         GList *l;
 
         g_debug ("Stopping wacom manager");
@@ -1492,7 +1741,7 @@ gsd_wacom_manager_stop (GsdWacomManager *manager)
                                 int id;
 
                                 id = get_device_id (l->data);
-                                grab_button (id, FALSE, manager->priv->screens);
+                                grab_button (id, FALSE, manager->priv->screen);
                         }
                 }
                 g_list_free (devices);
@@ -1500,14 +1749,11 @@ gsd_wacom_manager_stop (GsdWacomManager *manager)
                 p->device_manager = NULL;
         }
 
-        for (ls = p->screens; ls != NULL; ls = ls->next) {
-                gdk_window_remove_filter (gdk_screen_get_root_window (ls->data),
-                                          (GdkFilterFunc) filter_button_events,
-                                          manager);
-        }
+        gdk_window_remove_filter (gdk_screen_get_root_window (p->screen),
+                                  (GdkFilterFunc) filter_button_events,
+                                  manager);
 
-	for (l = p->rr_screens; l != NULL; l = l->next)
-		g_signal_handlers_disconnect_by_func (l->data, on_screen_changed_cb, manager);
+        g_signal_handlers_disconnect_by_func (p->rr_screen, on_screen_changed_cb, manager);
 
         g_clear_pointer (&p->osd_window, gtk_widget_destroy);
 }
@@ -1524,24 +1770,28 @@ gsd_wacom_manager_finalize (GObject *object)
 
         g_return_if_fail (wacom_manager->priv != NULL);
 
+        if (wacom_manager->priv->warned_devices) {
+                g_hash_table_destroy (wacom_manager->priv->warned_devices);
+                wacom_manager->priv->warned_devices = NULL;
+        }
+
         if (wacom_manager->priv->devices) {
                 g_hash_table_destroy (wacom_manager->priv->devices);
                 wacom_manager->priv->devices = NULL;
         }
 
-        if (wacom_manager->priv->screens != NULL) {
-                g_slist_free (wacom_manager->priv->screens);
-                wacom_manager->priv->screens = NULL;
-        }
-
-	if (wacom_manager->priv->rr_screens != NULL) {
-		g_list_free_full (wacom_manager->priv->rr_screens, g_object_unref);
-		wacom_manager->priv->rr_screens = NULL;
+	if (wacom_manager->priv->rr_screen != NULL) {
+		g_clear_object (&wacom_manager->priv->rr_screen);
+		wacom_manager->priv->rr_screen = NULL;
 	}
 
         if (wacom_manager->priv->start_idle_id != 0)
                 g_source_remove (wacom_manager->priv->start_idle_id);
 
+        if (wacom_manager->priv->notification_timeout_src_id != 0)
+                g_source_remove (wacom_manager->priv->notification_timeout_src_id);
+        remove_notification (wacom_manager);
+
         G_OBJECT_CLASS (gsd_wacom_manager_parent_class)->finalize (object);
 }
 
