@@ -374,7 +374,7 @@ is_laptop (GnomeRRScreen *screen, GnomeRROutputInfo *output)
 
         rr_output = gnome_rr_screen_get_output_by_name (screen, gnome_rr_output_info_get_name (output));
 
-        return gnome_rr_output_is_laptop (rr_output);
+        return gnome_rr_output_is_builtin_display (rr_output);
 }
 
 static GnomeRROutputInfo *
@@ -1048,16 +1048,16 @@ turn_on_and_get_rightmost_offset (GnomeRRScreen *screen, GnomeRROutputInfo *info
         return x;
 }
 
-/* Used from g_ptr_array_sort(); compares outputs based on their X position */
+/* Used from qsort(); compares outputs based on their X position */
 static int
-compare_output_positions (gconstpointer a, gconstpointer b)
+compare_output_positions (const void *a, const void *b)
 {
-        GnomeRROutputInfo **oa = (GnomeRROutputInfo **) a;
-        GnomeRROutputInfo **ob = (GnomeRROutputInfo **) b;
+        GnomeRROutputInfo *oa = (GnomeRROutputInfo *) a;
+        GnomeRROutputInfo *ob = (GnomeRROutputInfo *) b;
         int xa, xb;
 
-        gnome_rr_output_info_get_geometry (*oa, &xa, NULL, NULL, NULL);
-        gnome_rr_output_info_get_geometry (*ob, &xb, NULL, NULL, NULL);
+        gnome_rr_output_info_get_geometry (oa, &xa, NULL, NULL, NULL);
+        gnome_rr_output_info_get_geometry (ob, &xb, NULL, NULL, NULL);
 
         return xb - xa;
 }
@@ -1071,30 +1071,39 @@ static gboolean
 trim_rightmost_outputs_that_dont_fit_in_framebuffer (GnomeRRScreen *rr_screen, GnomeRRConfig *config)
 {
         GnomeRROutputInfo **outputs;
-        int i;
+        GnomeRROutputInfo **sorted_outputs;
+        int num_on_outputs;
+        int i, j;
         gboolean applicable;
-        GPtrArray *sorted_outputs;
 
         outputs = gnome_rr_config_get_outputs (config);
-        g_return_val_if_fail (outputs != NULL, FALSE);
 
         /* How many are on? */
 
-        sorted_outputs = g_ptr_array_new ();
+        num_on_outputs = 0;
         for (i = 0; outputs[i] != NULL; i++) {
                 if (gnome_rr_output_info_is_active (outputs[i]))
-                        g_ptr_array_add (sorted_outputs, outputs[i]);
+                        num_on_outputs++;
         }
 
         /* Lay them out from left to right */
 
-        g_ptr_array_sort (sorted_outputs, compare_output_positions);
+        sorted_outputs = g_new (GnomeRROutputInfo *, num_on_outputs);
+        j = 0;
+        for (i = 0; outputs[i] != NULL; i++) {
+                if (gnome_rr_output_info_is_active (outputs[i])) {
+                        sorted_outputs[j] = outputs[i];
+                        j++;
+                }
+        }
+
+        qsort (sorted_outputs, num_on_outputs, sizeof (sorted_outputs[0]), compare_output_positions);
 
         /* Trim! */
 
         applicable = FALSE;
 
-        for (i = sorted_outputs->len - 1; i >= 0; i--) {
+        for (i = num_on_outputs - 1; i >= 0; i--) {
                 GError *error = NULL;
                 gboolean is_bounds_error;
 
@@ -1108,13 +1117,13 @@ trim_rightmost_outputs_that_dont_fit_in_framebuffer (GnomeRRScreen *rr_screen, G
                 if (!is_bounds_error)
                         break;
 
-                gnome_rr_output_info_set_active (sorted_outputs->pdata[i], FALSE);
+                gnome_rr_output_info_set_active (sorted_outputs[i], FALSE);
         }
 
         if (config_is_all_off (config))
                 applicable = FALSE;
 
-        g_ptr_array_free (sorted_outputs, FALSE);
+        g_free (sorted_outputs);
 
         return applicable;
 }
@@ -2176,16 +2185,6 @@ gsd_xrandr_manager_finalize (GObject *object)
         G_OBJECT_CLASS (gsd_xrandr_manager_parent_class)->finalize (object);
 }
 
-static guint32
-clamp_timestamp (gint64 timestamp)
-{
-        if (timestamp < 0)
-                return 0;
-        if (timestamp > G_MAXUINT32)
-                return G_MAXUINT32;
-        return timestamp;
-}
-
 static void
 handle_method_call_xrandr_2 (GsdXrandrManager *manager,
                              const gchar *method_name,
@@ -2202,23 +2201,23 @@ handle_method_call_xrandr_2 (GsdXrandrManager *manager,
 
                 g_variant_get (parameters, "(xx)", &parent_window_id, &timestamp);
                 if (gsd_xrandr_manager_2_apply_configuration (manager, parent_window_id,
-                                                              clamp_timestamp(timestamp), &error) == FALSE) {
+                                                              timestamp, &error) == FALSE) {
                         g_dbus_method_invocation_return_gerror (invocation, error);
                 } else {
                         g_dbus_method_invocation_return_value (invocation, NULL);
                 }
         } else if (g_strcmp0 (method_name, "VideoModeSwitch") == 0) {
                 g_variant_get (parameters, "(x)", &timestamp);
-                gsd_xrandr_manager_2_video_mode_switch (manager, clamp_timestamp(timestamp), NULL);
+                gsd_xrandr_manager_2_video_mode_switch (manager, timestamp, NULL);
                 g_dbus_method_invocation_return_value (invocation, NULL);
         } else if (g_strcmp0 (method_name, "Rotate") == 0) {
                 g_variant_get (parameters, "(x)", &timestamp);
-                gsd_xrandr_manager_2_rotate (manager, clamp_timestamp(timestamp), NULL);
+                gsd_xrandr_manager_2_rotate (manager, timestamp, NULL);
                 g_dbus_method_invocation_return_value (invocation, NULL);
         } else if (g_strcmp0 (method_name, "RotateTo") == 0) {
                 GnomeRRRotation rotation;
                 g_variant_get (parameters, "(ix)", &rotation, &timestamp);
-                gsd_xrandr_manager_2_rotate_to (manager, rotation, clamp_timestamp(timestamp), NULL);
+                gsd_xrandr_manager_2_rotate_to (manager, rotation, timestamp, NULL);
                 g_dbus_method_invocation_return_value (invocation, NULL);
         }
 }
